/*
  This file has my implementation of the LAPACK routine dsyev
  for C++.  This program solves for the eigenvalues and, if
  desired, the eigenvectors for a square symmetric matrix H.
  It assumes that the upper triangle of H is stored.

  There are two function calls defined in this header, of the
  forms

    void dsyev(double **H, int n, double *E)
    void dsyev(double **H, int n, double *E, double **Evecs)

    H: the n by n symmetric matrix that we are solving, with the
       upper triangle stored
    n: the order of the square matrix H
    E: an n-element array to hold the eigenvalues of H
    Evecs: an n by n matrix to hold the eigenvectors of H, if
           they are requested.

  The function call is defined twice, so that whether or not
  eigenvectors are called for the proper version is called.

  Scot Shaw
  30 August 1999
*/

#ifndef __DSYEV_H__
#define __DSYEV_H__

namespace LapackLib {

void dsyev(double **H, int n, double *E);
void dsyev(double **H, int n, double *E, double **Evecs);

double *dsyev_ctof(double **in, int rows, int cols);
double **dsyev_ftoc(double *in, int rows, int cols);
void dsyev_ftoc2(double *in, double **out, int rows, int cols);
void dsyev_sort(double *E, double **Evecs, int N);

extern "C" void dsyev_(char *jobz, char *uplo, int *n, double *a,
		       int *lda, double *w, double *work, int *lwork,
		       int *info);

} // end namespace "LapackLib"

#endif
